from collections.abc import Generator

from _typeshed import Incomplete
from xmldiff import actions as actions

class Patcher:
    def patch(self, actions, tree): ...
    def handle_action(self, action, tree) -> None: ...

class DiffParser:
    def parse(self, diff) -> Generator[Incomplete, None, None]: ...
    def make_action(self, line): ...



from importlib.metadata import version as v
from typing import List

# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "Voltaire"
copyright = "2022, Ross Fenning"
author = "Ross Fenning"
release = v("voltaire")
version = ".".join(release.split(".")[:2])

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions: List[str] = []

templates_path = ["_templates"]
exclude_patterns: List[str] = []


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "sphinx_rtd_theme"
html_static_path = ["_static"]



from collections.abc import Generator

from _typeshed import Incomplete
from xmldiff import actions as actions
from xmldiff import utils as utils

class Differ:
    F: Incomplete
    uniqueattrs: Incomplete
    fast_match: Incomplete
    def __init__(
        self,
        F: Incomplete | None = ...,
        uniqueattrs: Incomplete | None = ...,
        ratio_mode: str = ...,
        fast_match: bool = ...,
    ) -> None: ...
    left: Incomplete
    right: Incomplete
    def clear(self) -> None: ...
    def set_trees(self, left, right) -> None: ...
    def append_match(self, lnode, rnode, max_match) -> None: ...
    def match(
        self, left: Incomplete | None = ..., right: Incomplete | None = ...
    ): ...
    def node_ratio(self, left, right): ...
    def node_text(self, node): ...
    def leaf_ratio(self, left, right): ...
    def child_ratio(self, left, right): ...
    def update_node_tag(
        self, left, right
    ) -> Generator[Incomplete, None, None]: ...
    def update_node_attr(
        self, left, right
    ) -> Generator[Incomplete, None, None]: ...
    def update_node_text(
        self, left, right
    ) -> Generator[Incomplete, None, None]: ...
    def find_pos(self, node): ...
    def align_children(
        self, left, right
    ) -> Generator[Incomplete, None, Incomplete]: ...
    def diff(
        self, left: Incomplete | None = ..., right: Incomplete | None = ...
    ) -> Generator[Incomplete, None, None]: ...



import re
from typing import List

import graphviz
from markdown.extensions import Extension
from markdown.preprocessors import Preprocessor
from sh import npx

DOT_START = re.compile(r"^```[\ \t]*dot[\ \t]*$")
DOT_END = re.compile(r"^```[\ \t]*$")


def generate_image(dot_code: str) -> str:
    """
    Converts Graphiz source code to SVG.
    :param dot_code: Dot input code as a string.
    :return: SVG code as a string.
    """
    src: graphviz.Source = graphviz.Source(dot_code)
    svg: str = src.pipe(format="svg").decode("utf-8")
    embeddable_svg = npx("-y", "svgo", "-", _in=svg)
    return embeddable_svg


class GraphvizProcessor(Preprocessor):
    def run(self, lines: List[str]) -> List[str]:
        def gen():
            looking_for_dots = True
            dot_code = ""

            for line in lines:
                if looking_for_dots:
                    if DOT_START.match(line):
                        looking_for_dots = False
                        continue
                    else:
                        yield line
                else:
                    if DOT_END.match(line):
                        yield generate_image(dot_code)
                        dot_code = ""
                        looking_for_dots = True
                        continue
                    else:
                        dot_code += line + "\n"
                        continue

        return list(gen())


class GraphvizExtension(Extension):
    def extendMarkdown(self, md):
        md.preprocessors.register(GraphvizProcessor(), "graphviz", 35)
        md.registerExtension(self)


def makeExtension(**kwargs):  # pragma: no cover
    return GraphvizExtension(**kwargs)



import datetime
import os
import threading
from textwrap import dedent

from invoke import task
from pelican import main as pelican_main
from pelican.settings import DEFAULT_CONFIG, get_settings_from_file

SETTINGS_FILE_BASE = "pelicanconf.py"


PUBLISH_FILE_BASE = "publishconf.py"
COMPOSE_YAML = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "docker-compose.verify.yml"
)
DOCKERFILE_VERIFY = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "Dockerfile.verify"
)


@task
def build(c):
    """Build local version of site"""
    c.run("npm install --save sass")
    pelican_main(["-s", SETTINGS_FILE_BASE])


@task
def publish(c, domain=None):
    """Build local version of site"""
    c.run("npm install --save sass")
    pelican_main(["-s", PUBLISH_FILE_BASE, "--output", "dist"])
    commit_message = "'Publish site on {}'".format(
        datetime.date.today().isoformat()
    )

    if domain:
        with open("dist/CNAME", "w") as f:
            f.write(domain)
        with open("dist/robots.txt", "w") as f:
            f.write(
                dedent(
                    """
                User-agent: *
                Disallow: /drafts/
                Sitemap: /sitemap.xml
            """
                )
            )
    c.run("ghp-import -b gh-pages " f"-m {commit_message} " "dist -p -f")


@task
def stage(c, domain=None):
    c.run("npm install --save sass")
    pelican_main(["-s", PUBLISH_FILE_BASE, "--output", "dist"])
    if domain:
        with open("dist/CNAME", "w") as f:
            f.write(domain)
        with open("dist/robots.txt", "w") as f:
            f.write(
                dedent(
                    f"""
                User-agent: *
                Disallow: /drafts/
                Sitemap: http://{domain}/sitemap.xml
            """
                )
            )
    c.run("docker run -v $PWD/dist:/usr/share/nginx/html -p 80:80 nginx")


@task
def verify(c, domain=None, opengraph=False, disqus=False):
    """
    Run checks locally on candidate site to publish.
    """
    c.run("npm install --save sass")
    pelican_main(["-s", PUBLISH_FILE_BASE, "--output", "dist"])
    if domain:
        with open("dist/CNAME", "w") as f:
            f.write(domain)
        with open("dist/robots.txt", "w") as f:
            f.write(
                dedent(
                    """
                User-agent: *
                Disallow: /drafts/
                Sitemap: /sitemap.xml
            """
                )
            )

    weblint_options = f"--set=OPENGRAPH={opengraph} --set=DISQUS={disqus}"
    try:
        c.run(
            f"""
            export DOMAIN={domain}
            export WEBLINT_OPTS='{weblint_options}'
            export DOCKERFILE_VERIFY='{DOCKERFILE_VERIFY}'
            docker-compose -f {COMPOSE_YAML} up --build --abort-on-container-exit --exit-code-from verify --renew-anon-volumes
            """
        )
    finally:
        c.run(
            f"""
            export DOMAIN={domain}
            export WEBLINT_OPTS='{weblint_options}'
            export DOCKERFILE_VERIFY='{DOCKERFILE_VERIFY}'
            docker-compose -f {COMPOSE_YAML} down --rmi local --remove-orphans
            """
        )


@task
def livereload(
    c, host="localhost", port=8000, test=True, opengraph=False, disqus=False
):
    """Automatically reload browser tab upon file modification."""
    c.run("npm install --save sass")
    from livereload import Server

    SETTINGS = {}
    SETTINGS.update(DEFAULT_CONFIG)
    LOCAL_SETTINGS = get_settings_from_file(SETTINGS_FILE_BASE)
    SETTINGS.update(LOCAL_SETTINGS)

    def run_weblint():
        c.run(
            f"scrapy weblint --set=OPENGRAPH={opengraph} --set=DISQUS={disqus} http://{host}:{port}/ &"
        )

    def build_test():
        build(c)
        if test:
            threading.Thread(target=run_weblint).start()

    build(c)
    server = Server()
    # Watch the base settings file
    server.watch(SETTINGS_FILE_BASE, build_test)
    # Watch content source files
    content_file_extensions = [".md", ".rst"]
    for extension in content_file_extensions:
        content_blob = "{}/**/*{}".format(SETTINGS["PATH"], extension)
        server.watch(
            content_blob,
            build_test,
            ignore=lambda f: f.endswith("cv.md"),
        )
    # Watch the theme's templates and static assets
    theme_path = SETTINGS["THEME"]
    server.watch(f"{theme_path}/templates/*.html", build_test)
    static_file_extensions = [".css", ".js"]
    for extension in static_file_extensions:
        static_file = f"{theme_path}/static/**/*{extension}"
        server.watch(static_file, build_test)
    # Serve output path on configured host and port
    server.serve(host=host, port=port, root=SETTINGS["OUTPUT_PATH"])



from xmldiff import main

from voltaire import graphviz


def test_generate_image():
    dot = """
    digraph g {
        A -> B;
    }
    """
    svg = graphviz.generate_image(dot)
    assert "DOCTYPE" not in svg
    expected = '<svg xmlns="http://www.w3.org/2000/svg" width="62pt" height="116pt" viewBox="0 0 62 116"><g class="graph" transform="translate(4 112)"><path fill="#fff" stroke="transparent" d="M-4 4v-116h62V4H-4z"/><g class="node"><ellipse cx="27" cy="-90" fill="none" stroke="#000" rx="27" ry="18"/><text x="27" y="-86.3" font-family="Times,serif" font-size="14" text-anchor="middle">A</text></g><g class="node"><ellipse cx="27" cy="-18" fill="none" stroke="#000" rx="27" ry="18"/><text x="27" y="-14.3" font-family="Times,serif" font-size="14" text-anchor="middle">B</text></g><g class="edge"><path fill="none" stroke="#000" d="M27-71.7v25.59"/><path stroke="#000" d="m30.5-46.1-3.5 10-3.5-10h7z"/></g></g></svg>'
    print(svg)
    diff = main.diff_texts(svg.encode("utf8"), expected.encode("utf8"))
    assert diff == []



from pytest_shell.fs import create_files

TASKS_PY = """import voltaire

namespace = voltaire.site()
"""

PAGE_MD = """title: My Page

This is my page.
"""

PELICAN_CONF = """from voltaire.pelican import *
"""


def test_single_page(shell, tmpdir):
    create_files(
        structure=[
            {"tasks.py": {"content": TASKS_PY}},
            {"pelicanconf.py": {"content": PELICAN_CONF}},
            {"content/page.md": {"content": PAGE_MD}},
        ],
        root=tmpdir,
    )

    result = shell.run("invoke", "build", cwd=tmpdir)
    pelican_output = result.stdout + result.stderr
    assert "ERROR" not in pelican_output
    assert "WARNING" not in pelican_output

    page_html = shell.run("cat", "output/page/index.html", cwd=tmpdir).stdout
    assert "<!DOCTYPE html>" in page_html
    assert "common.min.css" in page_html



from typing import NamedTuple

from _typeshed import Incomplete

class DeleteNode(NamedTuple):
    node: Incomplete

class InsertNode(NamedTuple):
    target: Incomplete
    tag: Incomplete
    position: Incomplete

class RenameNode(NamedTuple):
    node: Incomplete
    tag: Incomplete

class MoveNode(NamedTuple):
    node: Incomplete
    target: Incomplete
    position: Incomplete

class UpdateTextIn(NamedTuple):
    node: Incomplete
    text: Incomplete

class UpdateTextAfter(NamedTuple):
    node: Incomplete
    text: Incomplete

class UpdateAttrib(NamedTuple):
    node: Incomplete
    name: Incomplete
    value: Incomplete

class DeleteAttrib(NamedTuple):
    node: Incomplete
    name: Incomplete

class InsertAttrib(NamedTuple):
    node: Incomplete
    name: Incomplete
    value: Incomplete

class RenameAttrib(NamedTuple):
    node: Incomplete
    oldname: Incomplete
    newname: Incomplete

class InsertComment(NamedTuple):
    target: Incomplete
    position: Incomplete
    text: Incomplete



from doctrine import add_task
from invoke import Collection

from . import tasks

__ALL__ = ["site"]
TASKS = Collection("tasks")


def site(host="localhost", port=8000, domain=None):
    add_task(TASKS, tasks.build)
    add_task(TASKS, tasks.livereload, host=host, port=port)
    add_task(TASKS, tasks.publish, domain=domain)
    add_task(TASKS, tasks.stage, domain=domain)
    add_task(TASKS, tasks.verify, domain=domain)
    return TASKS



import os
from pathlib import Path
from typing import List, Optional

PATH = "content"
PAGE_PATHS = ["."]
ARTICLE_PATHS = ["posts"]
STATIC_PATHS: List[str] = []

SLUGIFY_SOURCE = "basename"

PAGE_URL = "{slug}/"
PAGE_SAVE_AS = "{slug}/index.html"
ARTICLE_URL = "{slug}/"
ARTICLE_SAVE_AS = "{slug}/index.html"

TIMEZONE = "Europe/London"

FEED_ALL_ATOM = None
CATEGORY_FEED_ATOM = None
AUTHOR_FEED_ATOM = None
AUTHOR_FEED_RSS = None
TRANSLATION_FEED_ATOM = None

PLUGINS = [
    "pelican.plugins.webassets",
    "pelican.plugins.sitemap",
    "voltaire.stats",
    "voltaire.wikilinks",
]
SITEMAP = {
    "format": "xml",
    "exclude": ["search/"],
}

THEME = os.path.join(os.path.dirname(os.path.realpath(__file__)), "theme")
THEME_STATIC_PATHS = [
    os.path.join(THEME, "static"),
    os.path.join(os.getcwd(), "static"),
]
WEBASSETS_CONFIG = [
    ("SASS_LOAD_PATHS", [str(Path.cwd() / "node_modules")]),
    ("SASS_BIN", str(Path.cwd() / "node_modules" / ".bin" / "sass")),
    ("SASS_USE_SCSS", True),
]

# LOCAL_SCSS: Optional[str] = os.path.join(os.getcwd(), "static", "css", "style.scss")
LOCAL_SCSS: Optional[str] = None
path: Path = Path.cwd() / "static" / "css" / "style.scss"
if path.exists():
    LOCAL_SCSS = str(path)

MARKDOWN = {
    "extension_configs": {
        "voltaire.mermaid": {},
        "plantuml_markdown": {},
        "markdown.extensions.extra": {},
        "markdown.extensions.toc": {},
    },
    "output_format": "html5",
}

TYPOGRIFY = True



from _typeshed import Incomplete

class diff_match_patch:
    Diff_Timeout: float
    Diff_EditCost: int
    Match_Threshold: float
    Match_Distance: int
    Patch_DeleteThreshold: float
    Patch_Margin: int
    Match_MaxBits: int
    def __init__(self) -> None: ...
    DIFF_DELETE: int
    DIFF_INSERT: int
    DIFF_EQUAL: int
    def diff_main(
        self,
        text1,
        text2,
        checklines: bool = ...,
        deadline: Incomplete | None = ...,
    ): ...
    def diff_compute(self, text1, text2, checklines, deadline): ...
    def diff_lineMode(self, text1, text2, deadline): ...
    def diff_bisect(self, text1, text2, deadline): ...
    def diff_bisectSplit(self, text1, text2, x, y, deadline): ...
    def diff_linesToChars(self, text1, text2): ...
    def diff_charsToLines(self, diffs, lineArray) -> None: ...
    def diff_commonPrefix(self, text1, text2): ...
    def diff_commonSuffix(self, text1, text2): ...
    def diff_commonOverlap(self, text1, text2): ...
    def diff_halfMatch(self, text1, text2): ...
    def diff_cleanupSemantic(self, diffs) -> None: ...
    def diff_cleanupSemanticLossless(self, diffs): ...
    BLANKLINEEND: Incomplete
    BLANKLINESTART: Incomplete
    def diff_cleanupEfficiency(self, diffs) -> None: ...
    def diff_cleanupMerge(self, diffs) -> None: ...
    def diff_xIndex(self, diffs, loc): ...
    def diff_prettyHtml(self, diffs): ...
    def diff_text1(self, diffs): ...
    def diff_text2(self, diffs): ...
    def diff_levenshtein(self, diffs): ...
    def diff_toDelta(self, diffs): ...
    def diff_fromDelta(self, text1, delta): ...
    def match_main(self, text, pattern, loc): ...
    def match_bitap(self, text, pattern, loc): ...
    def match_alphabet(self, pattern): ...
    def patch_addContext(self, patch, text) -> None: ...
    def patch_make(
        self, a, b: Incomplete | None = ..., c: Incomplete | None = ...
    ): ...
    def patch_deepCopy(self, patches): ...
    def patch_apply(self, patches, text): ...
    def patch_addPadding(self, patches): ...
    def patch_splitMax(self, patches) -> None: ...
    def patch_toText(self, patches): ...
    def patch_fromText(self, textline): ...

class patch_obj:
    diffs: Incomplete
    start1: Incomplete
    start2: Incomplete
    length1: int
    length2: int
    def __init__(self) -> None: ...
